import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter


class SpreeApiV1(object):
  def __init__(self, endpoint, api_key):
    self.endpoint = endpoint
    self.api_key = api_key

    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[429, 500, 502, 503, 504],
    )
    self.session = requests.Session()
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))

  def import_product(self, product_data):
    endpoint_url = self.append_api_key(self.endpoint + '/api/v1/products/import')

    return self.session.post(endpoint_url, json=product_data)

  def append_api_key(self, url):
    separator = '&' if '?' in url else '?'
    return url + '%stoken=%s' % (separator, self.api_key)
