import datetime
from urllib3.util import Retry

import requests
from requests.adapters import HTTPAdapter


class SpreeApiV2(object):
  def __init__(self, endpoint, client_id, client_secret):
    self.endpoint = endpoint
    self.client_id = client_id
    self.client_secret = client_secret
    self.token = None

    retries = Retry(
      total=7,
      backoff_factor=0.1,
      status_forcelist=[429, 502, 503, 504],
    )
    self.session = requests.Session()
    self.session.mount(self.endpoint, HTTPAdapter(max_retries=retries))
    self.token = self.authorize()

  def authorize(self):
    url = '{}/spree_oauth/token'.format(self.endpoint)
    payload = {
      'grant_type': 'client_credentials',
      'client_id': self.client_id,
      'client_secret': self.client_secret
    }

    resp = self.session.post(url, json=payload, headers=self.dft_headers)

    return resp.json()

  def dft_headers(self):
    return {
      'Content-Type': 'application/json',
      'Accept': 'application/json'
    }
