import datetime

from shipstation.api import *
from .models import *

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *
from em_tasks.store.product_source import ProductSource


class StoreUtil():
  stores = dict()
  sellers = dict()
  gmc_stores = dict()
  ORDER_ITEM_STATUSES = {
    'PENDING': 0,
    'ORDERED': 1,
    'SHIPPED': 2,
    'DELIVERED': 3,
    'CANCELED': 4,
    'REQUEST_CANCEL': 6,
    'REQUEST_RETURN': 7,
    'CANCEL_REQUEST_APPROVED': 8,
    'CANCEL_REQUEST_DECLINED': 9,
    'RETURN_REQUEST_APPROVED': 10,
    'RETURN_REQUEST_DECLINED': 11,
    'RETURNED': 12,
  }

  def __init__(self):
    pass

  @classmethod
  def get_store_by_code(cls, store_code):
    if store_code not in cls.stores:
      cls.stores[store_code] = Store.get_or_none(Store.code == store_code)

    return cls.stores[store_code]

  @classmethod
  def get_seller(cls, seller_id):
    if seller_id not in cls.sellers:
      cls.sellers[seller_id] = StoreSeller.get_or_none(StoreSeller.id == seller_id)

    return cls.sellers.get(seller_id, None)

  @classmethod
  def get_google_shopping_store(cls, merchant_id=None):
    if not merchant_id:
      merchant_id = '654568556'

    if merchant_id not in cls.gmc_stores:
      cls.gmc_stores[merchant_id] = GmcStore.get_or_none(GmcStore.merchant_id == merchant_id)

    if merchant_id not in cls.gmc_stores or not cls.gmc_stores[merchant_id]:
      return

    return GoogleShopping(merchant_id, json.loads(cls.gmc_stores[merchant_id].service_account_data))

  @classmethod
  def save_store_product(cls, seller_id, source, source_product_id, product_attrs):
    s_product_id = product_attrs.pop('s_product_id', None)
    if not s_product_id:
      return

    store_product = StoreProduct.get_or_none(StoreProduct.s_product_id == s_product_id)
    if not store_product:
      store_product = StoreProduct(s_product_id=s_product_id)

    data = {
      'store_seller_id': seller_id,
      'source': source,
      'source_product_id': source_product_id,
    }
    data.update(product_attrs)
    for k, v in data.items():
      if v:
        setattr(store_product, k, v)
    store_product.save()

    for gp in GmcProduct.select().where(GmcProduct.handle == store_product.handle).execute():
      if gp.store_product_id != store_product.id:
        gp.store_product_id = store_product.id
        gp.save()

  @classmethod
  def delete_store_product(cls, product_id):
    StoreProduct.delete().where(StoreProduct.s_product_id == product_id).execute()

  @classmethod
  def get_amz_products(cls, marketplace):
    page = 1
    batch_size = 2500
    source = 'AMZ_{}'.format(marketplace.upper())
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'asin': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_ebay_products(cls, marketplace):
    page = 1
    batch_size = 2500
    source = 'Ebay_{}'.format(marketplace.upper())
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_rakuten_products(cls):
    page = 1
    batch_size = 2500
    source = 'Rakuten'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_naver_products(cls):
    page = 1
    batch_size = 2500
    source = 'Naver'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_tw_products(cls):
    page = 1
    batch_size = 2500
    source = 'www_books_com_tw'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_dangdang_products(cls):
    page = 1
    batch_size = 2500
    source = 'Dangdang'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle']}

      page += 1

  @classmethod
  def get_coupang_products(cls):
    page = 1
    batch_size = 2500
    source = 'Coupang'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'store_seller_id': sp['store_seller_id'], 'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle'], 'title': sp['title']}

      page += 1

  @classmethod
  def get_collective_products(cls):
    page = 1
    batch_size = 2500
    source = 'Collective'
    query = StoreProduct.select(StoreProduct.id, StoreProduct.store_seller_id, StoreProduct.s_product_id, StoreProduct.source, StoreProduct.source_product_id, StoreProduct.handle).where(StoreProduct.source == source)
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      products = dict()
      for sp in query.paginate(page, batch_size).dicts():
        yield {'product_id': sp['s_product_id'], 'source': source, 'source_product_id': sp['source_product_id'], 'handle': sp['handle'], 'title': sp['title']}

      page += 1

  @classmethod
  def update_daily_statistics(cls, days=7):
    date_format = '%Y-%m-%d'

    today = datetime.datetime.utcnow()
    from_date = (today - datetime.timedelta(days=days)).date()

    order_events = dict()
    # for order_event in OrderEvent.select().where((OrderEvent.event_time >= from_date) & (OrderEvent.event_type == 'OrderSales')).execute():
    for order_event in OrderEvent.select().where(OrderEvent.event_time >= from_date).execute():
      if order_event.order_item_id not in order_events:
        order_events[order_event.order_item_id] = []

      order_events[order_event.order_item_id].append(order_event)

    order_events_by_seller = dict()
    order_events_by_source = dict()
    order_events_all = []
    order_item_ids = list(order_events.keys())
    order_items = OrderItem.select().where(OrderItem.id.in_(order_item_ids)).execute()
    for order_item in order_items:
      if order_item.id not in order_events:
        continue

      order_event = order_events[order_item.id]
      order_events_all.extend(order_event)

      seller_id = order_item.vendor_id
      # if seller_id not in order_events_by_seller:
      #   order_events_by_seller[seller_id] = dict()
      # order_events_by_seller[seller_id][order_item.id] = order_event
      if seller_id not in order_events_by_seller:
        order_events_by_seller[seller_id] = []
      order_events_by_seller[seller_id].extend(order_event)

      source = order_item.source
      # if source not in order_events_by_source:
      #   order_events_by_source[source] = dict()
      # order_events_by_source[source][order_item.id] = order_event
      if source not in order_events_by_source:
        order_events_by_source[source] = []
      order_events_by_source[source].extend(order_event)

    cls.aggregate_forwarding_daily_statistics(from_date)
    # oes = []
    # for items in order_events.values():
    #   oes.extend(items)
    cls.aggregate_order_events(order_events_all, from_date, 'Date')

    for source, order_events_s in order_events_by_source.items():
      cls.aggregate_order_events(order_events_s, from_date, 'Source', source)

    for seller_id, order_events_s in order_events_by_seller.items():
      cls.aggregate_order_events(order_events_s, from_date, 'Seller', seller_id)

    cls.aggregate_google_ads_daily_statistics(from_date)
    cls.aggregate_bing_ads_daily_statistics(from_date)

  @classmethod
  def aggregate_forwarding_daily_statistics(cls, from_date):
    date_format = '%Y-%m-%d'

    forwardings_stats = dict()
    for forwarding in SellerForwarding.select().where(SellerForwarding.created_at >= from_date).execute():
      date_s = forwarding.created_at.strftime(date_format)
      if date_s not in forwardings_stats:
        forwardings_stats[date_s] = dict()

      sid = forwarding.store_seller_id
      if sid not in forwardings_stats[date_s]:
        forwardings_stats[date_s][sid] = 0

      forwardings_stats[date_s][sid] += forwarding.amount

    d = dict()
    forwardings_stats_for_seller = dict(forwardings_stats)
    for ds in SellerDailyStatistic.select().where((SellerDailyStatistic.date >= from_date) & (SellerDailyStatistic.label == 'Forwarding')).execute():
      date_s = ds.date.strftime(date_format)
      daily_stats = forwardings_stats_for_seller.get(date_s, None)
      if not daily_stats or ds.vendor_id not in daily_stats or not daily_stats[ds.vendor_id]:
        continue

      seller_stat = daily_stats.pop(ds.vendor_id, None)
      amount_db = round(ds.amount, 2)
      amount_now = round(seller_stat, 2)
      if amount_db != amount_now:
        ds.amount = amount_now
        ds.save()

    try:
      forwardings_stats_sds_records = []
      for date_s, seller_stats in forwardings_stats_for_seller.items():
        for seller_id, seller_stat in seller_stats.items():
          forwardings_stats_sds_records.append({
            'vendor_id': seller_id,
            'date': date_s,
            'amount': seller_stat,
            'label': 'Forwarding'
          })
      SellerDailyStatistic.insert_many(forwardings_stats_sds_records).execute()
    except Exception as e:
      logger.exception(e)

    for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == 'Forwarding')).execute():
      date_s = ds.date.strftime(date_format)
      seller_stats = forwardings_stats.pop(date_s, None)
      if not seller_stats:
        continue

      forwarding_cost = 0
      for seller_stat in seller_stats.values():
        forwarding_cost += seller_stat

      amount_db = round(ds.amount, 2)
      amount_now = round(forwarding_cost, 2)
      if amount_db != amount_now:
        ds.amount = amount_now
        ds.save()

    if forwardings_stats:
      forwardings_ds_records = []
      for date_s, seller_stats in forwardings_stats.items():
        forwarding_cost = 0
        for seller_stat in seller_stats.values():
          forwarding_cost += seller_stat

        forwardings_ds_records.append({
          'date': date_s,
          'amount': round(forwarding_cost, 2),
          'label': 'Forwarding'
        })
      DailyStatistic.insert_many(forwardings_ds_records).execute()

  @classmethod
  def aggregate_google_ads_daily_statistics(cls, from_date):
    date_format = '%Y-%m-%d'

    google_ads_stats = dict()
    for google_ads_insight in GoogleAdsInsight.select().where(GoogleAdsInsight.date >= from_date).execute():
      date_s = google_ads_insight.date.strftime(date_format)
      if date_s not in google_ads_stats:
        google_ads_stats[date_s] = 0

      google_ads_stats[date_s] += google_ads_insight.spend

    for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == 'GoogleAds')).execute():
      date_s = ds.date.strftime(date_format)
      amount = google_ads_stats.pop(date_s, None)
      if amount is None:
        continue

      amount_db = round(ds.amount, 2)
      amount_now = round(amount, 2)
      if amount_db != amount_now:
        ds.amount = amount_now
        ds.save()
    if google_ads_stats:
      google_ads_ds_records = []
      for date_s, amount in google_ads_stats.items():
        google_ads_ds_records.append({
          'date': date_s,
          'amount': round(amount, 2),
          'label': 'GoogleAds'
        })
      DailyStatistic.insert_many(google_ads_ds_records).execute()

  @classmethod
  def aggregate_bing_ads_daily_statistics(cls, from_date):
    date_format = '%Y-%m-%d'

    bing_ads_stats = dict()
    for bing_ads_insight in BingAdsInsight.select().where(BingAdsInsight.date >= from_date).execute():
      date_s = bing_ads_insight.date.strftime(date_format)
      if date_s not in bing_ads_stats:
        bing_ads_stats[date_s] = 0

      bing_ads_stats[date_s] += bing_ads_insight.spend

    for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == 'BingAds')).execute():
      date_s = ds.date.strftime(date_format)
      amount = bing_ads_stats.pop(date_s, None)
      if amount is None:
        continue

      amount_db = round(ds.amount, 2)
      amount_now = round(amount, 2)
      if amount_db != amount_now:
        ds.amount = amount_now
        ds.save()
    if bing_ads_stats:
      bing_ads_ds_records = []
      for date_s, amount in bing_ads_stats.items():
        bing_ads_ds_records.append({
          'date': date_s,
          'amount': round(amount, 2),
          'label': 'BingAds'
        })
      DailyStatistic.insert_many(bing_ads_ds_records).execute()

  @classmethod
  def aggregate_order_events(cls, order_events, from_date, agg_type='Date', identifier=None):
    date_format = '%Y-%m-%d'

    order_stats = {
      'Sales': {},
      'Tax': {},
      'ShippingCharge': {},
      'Discount': {},
      'Orders': {},
      'Items': {},
      'Cancel': {},
      'Refund': {},
      'Product': {},
      'Shipping': {},
      'CancelCnt': {},
      'RefundCnt': {}
    }
    for order_event in order_events:
      date_s = order_event.event_time.strftime(date_format)
      for k in order_stats.keys():
        if date_s not in order_stats[k]:
          order_stats[k][date_s] = 0

      data = json.loads(order_event.event_data)
      if order_event.event_type == 'OrderSales':
        order_stats['Orders'][date_s] += 1
        order_stats['Sales'][date_s] += data['amount']
        order_stats['Tax'][date_s] += data['tax']
        # TODO: Fix ShippingCharge calculation
        order_stats['ShippingCharge'][date_s] += data['shipping_charge']
        order_stats['Discount'][date_s] += data['discount']
        order_stats['Items'][date_s] += data['quantity']
        continue

      if order_event.event_type == 'OrderCancel':
        order_stats['Cancel'][date_s] += data.get('amount', 0)
        order_stats['CancelCnt'][date_s] += 1
        continue

      if order_event.event_type == 'OrderRefund':
        order_stats['Refund'][date_s] += data.get('amount', 0)
        order_stats['RefundCnt'][date_s] += 1
        continue

      k = None
      if order_event.event_type == 'OrderPurchased':
        k = 'Product'
      elif order_event.event_type == 'OrderShipped':
        k = 'Shipping'
      else:
        # TODO: Add more order event support
        pass

      if not k:
        continue

      order_stats[k][date_s] += data.get('amount', 0)

    model = DailyStatistic
    identifier_type = None
    if agg_type == 'Seller':
      if not identifier:
        return
      model = SellerDailyStatistic
      identifier_type = 'vendor_id'
    elif agg_type == 'Source':
      if not identifier:
        return
      model = SourceDailyStatistic
      identifier_type = 'source'

    for label, stat in order_stats.items():
      qeury = None
      if identifier_type is None:
        query = DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == label))
      elif identifier_type == 'vendor_id':
        query = SellerDailyStatistic.select().where((SellerDailyStatistic.date >= from_date) & (SellerDailyStatistic.label == label) & (SellerDailyStatistic.vendor_id == identifier))
      elif identifier_type == 'source':
        query = SourceDailyStatistic.select().where((SourceDailyStatistic.date >= from_date) & (SourceDailyStatistic.label == label) & (SourceDailyStatistic.source == identifier))

      for ds in query.execute():
        date_s = ds.date.strftime(date_format)
        if date_s not in stat:
          continue

        amount = stat.pop(date_s, None)
        if amount is None:
          continue

        amount_db = round(ds.amount, 2)
        amount_now = round(amount, 2)
        if amount_db != amount_now:
          ds.amount = amount_now
          ds.save()

      if stat:
        ds_records = []
        for date_s, amount in stat.items():
          record = {
            'date': date_s,
            'amount': round(amount, 2),
            'label': label
          }
          if identifier:
            record[identifier_type] = identifier
          ds_records.append(record)
        model.insert_many(ds_records).execute()

      logger.info('[OrderEventAggregate] Type: %s, Label: %s', agg_type, label)

  # @classmethod
  # def update_daily_statistics(cls, days=7):
  #   date_format = '%Y-%m-%d'

  #   today = datetime.datetime.utcnow()
  #   from_date = (today - datetime.timedelta(days=days)).date()
  #   order_stats = {'Sales': {}, 'Tax': {}, 'ShippingCharge': {}, 'Discount': {}, 'Orders': {}, 'Items': {}}
  #   for order_event in OrderEvent.select().where((OrderEvent.event_time >= from_date) & (OrderEvent.event_type == 'OrderSales')).execute():
  #     date_s = order_event.event_time.strftime(date_format)
  #     for k in order_stats.keys():
  #       if date_s not in order_stats[k]:
  #         order_stats[k][date_s] = 0

  #     data = json.loads(order_event.event_data)
  #     order_stats['Orders'][date_s] += 1
  #     order_stats['Sales'][date_s] += data['amount']
  #     order_stats['Tax'][date_s] += data['tax']
  #     order_stats['ShippingCharge'][date_s] += data['shipping_charge']
  #     order_stats['Discount'][date_s] += data['discount']
  #     order_stats['Items'][date_s] += data['quantity']

  #   for label, stat in order_stats.items():
  #     for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == label)).execute():
  #       date_s = ds.date.strftime(date_format)
  #       if date_s not in stat:
  #         continue

  #       amount = stat.pop(date_s, None)
  #       if amount is None:
  #         continue

  #       amount_db = round(ds.amount, 2)
  #       amount_now = round(amount, 2)
  #       if amount_db != amount_now:
  #         ds.amount = amount_now
  #         ds.save()

  #     if stat:
  #       ds_records = []
  #       for date_s, amount in stat.items():
  #         ds_records.append({
  #           'date': date_s,
  #           'amount': round(amount, 2),
  #           'label': label
  #         })
  #       DailyStatistic.insert_many(ds_records).execute()

  #   order_fees = {'OrderCancel': 'Cancel', 'OrderRefund': 'Refund', 'OrderPurchased': 'Product', 'OrderShipped': 'Shipping'}
  #   for event_type, label in order_fees.items():
  #     order_stats = dict()
  #     for order_event in OrderEvent.select().where((OrderEvent.event_time >= from_date) & (OrderEvent.event_type == event_type)).execute():
  #       date_s = order_event.event_time.strftime(date_format)
  #       if date_s not in order_stats:
  #         order_stats[date_s] = 0

  #       data = json.loads(order_event.event_data)
  #       order_stats[date_s] += data.get('amount', 0)
  #     for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == label)).execute():
  #       date_s = ds.date.strftime(date_format)
  #       amount = order_stats.pop(date_s, None)
  #       if amount is None:
  #         continue

  #       amount_db = round(ds.amount, 2)
  #       amount_now = round(amount, 2)
  #       if amount_db != amount_now:
  #         ds.amount = amount_now
  #         ds.save()

  #     if order_stats:
  #       order_ds_records = []
  #       for date_s, amount in order_stats.items():
  #         order_ds_records.append({
  #           'date': date_s,
  #           'amount': round(amount, 2),
  #           'label': label
  #         })
  #       DailyStatistic.insert_many(order_ds_records).execute()

  #     logger.info('[DailyStatistic] Label: %s', label)

  #   google_ads_stats = dict()
  #   for google_ads_insight in GoogleAdsInsight.select().where(GoogleAdsInsight.date >= from_date).execute():
  #     date_s = google_ads_insight.date.strftime(date_format)
  #     if date_s not in google_ads_stats:
  #       google_ads_stats[date_s] = 0

  #     google_ads_stats[date_s] += google_ads_insight.spend

  #   for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == 'GoogleAds')).execute():
  #     date_s = ds.date.strftime(date_format)
  #     amount = google_ads_stats.pop(date_s, None)
  #     if amount is None:
  #       continue

  #     amount_db = round(ds.amount, 2)
  #     amount_now = round(amount, 2)
  #     if amount_db != amount_now:
  #       ds.amount = amount_now
  #       ds.save()
  #   if google_ads_stats:
  #     google_ads_ds_records = []
  #     for date_s, amount in google_ads_stats.items():
  #       google_ads_ds_records.append({
  #         'date': date_s,
  #         'amount': round(amount, 2),
  #         'label': 'GoogleAds'
  #       })
  #     DailyStatistic.insert_many(google_ads_ds_records).execute()

  #   bing_ads_stats = dict()
  #   for bing_ads_insight in BingAdsInsight.select().where(BingAdsInsight.date >= from_date).execute():
  #     date_s = bing_ads_insight.date.strftime(date_format)
  #     if date_s not in bing_ads_stats:
  #       bing_ads_stats[date_s] = 0

  #     bing_ads_stats[date_s] += bing_ads_insight.spend

  #   for ds in DailyStatistic.select().where((DailyStatistic.date >= from_date) & (DailyStatistic.label == 'BingAds')).execute():
  #     date_s = ds.date.strftime(date_format)
  #     amount = bing_ads_stats.pop(date_s, None)
  #     if amount is None:
  #       continue

  #     amount_db = round(ds.amount, 2)
  #     amount_now = round(amount, 2)
  #     if amount_db != amount_now:
  #       ds.amount = amount_now
  #       ds.save()
  #   if bing_ads_stats:
  #     bing_ads_ds_records = []
  #     for date_s, amount in bing_ads_stats.items():
  #       bing_ads_ds_records.append({
  #         'date': date_s,
  #         'amount': round(amount, 2),
  #         'label': 'BingAds'
  #       })
  #     DailyStatistic.insert_many(bing_ads_ds_records).execute()

  @classmethod
  def convert_order_item_to_shipstation_order(self, order_item):
    order_key = None
    order_number = '{}-{}'.format(order_item.s_order_number, order_item.s_line_item_id)
    ss_order = ShipStationOrder(order_key, order_number)
    ss_order.set_order_date(order_item.s_purchase_date)

    status = 'awaiting_shipment'
    if order_item.canceled_at or order_item.refund_qty > 0:
      status = 'cancelled'
    elif order_item.s_tracking and not order_item.s_tracking.startswith('CP'):
      status = 'shipped'

    ss_order.set_status(status)

    name = '{} {}'.format(order_item.first_name, order_item.last_name)
    ss_order.set_customer_details(name, order_item.email)

    shipping_address = ShipStationAddress(
      name, order_item.company, order_item.address1, order_item.address2, None,
      order_item.city, order_item.state, order_item.zipcode, order_item.country,
      order_item.phone)
    ss_order.set_shipping_address(shipping_address)

    billing_address = shipping_address
    if order_item.billing_address:
      try:
        ba = json.loads(order_item.billing_address)
        billing_address = ShipStationAddress(
          '{} {}'.format(ba['first_name'], ba['last_name']), ba['company'],
          ba['address1'], ba['address2'], None, ba['city'], ba['state'], ba['zipcode'],
          ba['country'], ba['phone'])
      except:
        pass
    ss_order.set_billing_address(billing_address)

    image_url = order_item.s_image_url
    if order_item.s_image_url and not order_item.s_image_url.startswith('http'):
      image_url = 'https://everymarket.com{}'.format(order_item.s_image_url)
    item = ShipStationItem(
      order_item.s_line_item_id, order_item.s_sku, order_item.s_name, image_url,
      order_item.s_quantity, order_item.s_price)
    ss_order.add_item(item)

    ss_order.amount_paid = order_item.s_total
    ss_order.tax_amount = order_item.s_tax
    ss_order.shipping_amount = order_item.s_shipping_charge
    ss_order.requested_shipping_service = order_item.s_shipping_method
    ss_order.internal_notes = order_item.note

    return ss_order

  @classmethod
  def convert_order_item_to_sheet_order(self, order_item):
    if not order_item.source:
      source_product_url = order_item.s_url
    elif order_item.source.startswith('EM'):
      source_product_url = order_item.s_url
    else:
      source_product_url = ProductSource.get_source_url(order_item.source, order_item.source_id)

    remark = ''
    if order_item.canceled:
      remark = 'Cancelled'
    if order_item.s_removed:
      remark = 'Removed'
    elif order_item.refund_qty:
      if order_item.refund_qty == order_item.s_quantity:
        remark = 'Refunded'
      else:
        remark = 'Partial Refunded'
    else:
      if not order_item.s_product_id or not order_item.s_variant_id:
        remark = 'ProductDeleted'
    return {
      'status': 'n',
      'order-id': '{}-{}'.format(order_item.s_order_id, order_item.s_line_item_id),
      'order-link': 'https://admin.shopify.com/store/d83075/orders/{}'.format(order_item.s_order_id),
      'recipient-name': '{} {}'.format(order_item.first_name, order_item.last_name),
      'purchase-date': order_item.s_purchase_date.isoformat(),
      'sku address': order_item.s_url,
      'sku': '{}-{}'.format(order_item.s_order_number, order_item.s_sku),
      'price': order_item.s_price - (order_item.s_discount / order_item.s_quantity),
      'quantity-purchased': order_item.s_quantity - order_item.refund_qty,
      'shipping-fee': order_item.s_shipping_charge,
      'ship-state': order_item.state,
      'isbn-address': source_product_url,
      'isbn': order_item.source_id,
      'SELLER': '',
      'SELLER-ID': '',
      'SELLER PRICE': '',
      'URL': '',
      'condition': 'new',
      'character': '',
      'Remark': remark,
      'reference': '',
      'CODE': '',
      '0': '',
      'item-name': order_item.s_name,
      'ship-address-1': order_item.address1,
      'ship-address-2': order_item.address2 or '',
      'ship-city': order_item.city,
      'ship-zip': order_item.zipcode,
      'ship-phone-number': order_item.phone or '',
      'COST': '',
      'ORDER NUMBER': '',
      'ACCOUNT': '',
      'last code': '',
      'ship-country': order_item.country,
      'SID': '',
      'Sales Channel': '',
      'Condition': 'New-New',
      'Shipping Service': order_item.s_shipping_method or '',
      'Expected Ship Date': order_item.s_min_delivery_date or '',
      'Estimated Delivery Date': order_item.s_min_delivery_date or '',
      'Order Date': '',
      'Supplier Estimated Delivery Date': '',
      'Carrier': '',
      'Tracking Number': '',
      'Note1': '',
      'Note2': '',
      'Note3': '',
    }

  @classmethod
  def update_order_items_status(cls, order_item_ids):
    ignore_codes = [
      cls.ORDER_ITEM_STATUSES['REQUEST_CANCEL'],
      cls.ORDER_ITEM_STATUSES['CANCEL_REQUEST_APPROVED'],
      cls.ORDER_ITEM_STATUSES['CANCEL_REQUEST_DECLINED'],
      cls.ORDER_ITEM_STATUSES['REQUEST_RETURN'],
      cls.ORDER_ITEM_STATUSES['RETURN_REQUEST_APPROVED'],
      cls.ORDER_ITEM_STATUSES['RETURN_REQUEST_DECLINED'],
      cls.ORDER_ITEM_STATUSES['RETURNED'],
      cls.ORDER_ITEM_STATUSES['DELIVERED'],
    ]

    fulfillments = dict()
    for fulfillment in FulfillOrder.select().where(FulfillOrder.order_item_id.in_(order_item_ids)).execute():
      fulfillments[fulfillment.order_item_id] = fulfillment

    for order_item in OrderItem.select().where(OrderItem.id.in_(order_item_ids)).execute():
      status_current = order_item.status
      if order_item.canceled:
        order_item.status = cls.ORDER_ITEM_STATUSES['CANCELED']
      elif order_item.status in ignore_codes:
        pass
      elif order_item.s_tracking:
        order_item.status = cls.ORDER_ITEM_STATUSES['SHIPPED']
      elif order_item.id in fulfillments:
        order_item.status = cls.ORDER_ITEM_STATUSES['ORDERED']

      if order_item.status != status_current:
        order_item.save()

  @classmethod
  def update_customer_returns(cls):
    customer_returns = dict()
    for customer_return in CustomerReturn.select().where(CustomerReturn.status == 0).execute():
      customer_returns[customer_return.tracking_number] = customer_return

    tracking_numbers = list(customer_returns.keys())
    for tracking_timeline in TrackingTimeline.select().where(TrackingTimeline.tracking_number.in_(tracking_numbers)).execute():
      if not tracking_timeline.timelines:
        continue

      tracking_number = tracking_timeline.tracking_number
      if tracking_number not in customer_returns:
        continue

      customer_return = customer_returns[tracking_number]
      timelines = json.loads(tracking_timeline.timelines)
      not_sent = None
      if len(timelines) < 1:
        not_sent = tracking_timeline.timelines.lower().find('cancel') != -1
      else:
        timeline = timelines[0]
        s = '{}, {}'.format(timeline.get('status', ''), timeline.get('status_detail', '')).lower()
        for k in ['awaiting item', 'label created', 'label', 'has not received the package yet', 'pre-shipment']:
          if s.find(k) != -1:
            not_sent = True
            break

      if not_sent:
        continue

      customer_return.status = 1
      customer_return.save()

  def save_product_seller(self, store_id, product_id, seller_id):
    pass

  def is_in_gmc(self, store_id, product):
    pass

  def upload_to_gmc(self, store_id, product):
    pass
