import datetime
import json
import datetime
import pdb

from shopping_utils.bing_shopping import BingShopping
from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *


class GmcToBmcTask():
  def __init__(self, bing_merchant_id):
    ba_account = BingAdsAccount.get_or_none(BingAdsAccount.merchant_id == bing_merchant_id)
    self.active = ba_account and ba_account.active
    if not self.active:
      return

    merchant_id = ba_account.merchant_id
    client_id = ba_account.client_id
    developer_token = ba_account.developer_token
    refresh_token = ba_account.refresh_token

    self.bs = BingShopping(merchant_id, client_id, developer_token, refresh_token)
    now = datetime.datetime.now()
    expiration_date = now + datetime.timedelta(days=30)
    self.expire_str = expiration_date.strftime("%Y-%m-%dT%H:%M:%SZ")

  def run(self):
    for gmc_store in GmcStore.select().order_by(GmcStore.id.desc()).execute():
      if not gmc_store.active:
        continue

      merchant_id = gmc_store.merchant_id

      gs = None
      try:
        gs = GoogleShopping(merchant_id, json.loads(gmc_store.service_account_data))
      except Exception as e:
        logger.exception(e)

      if not gs:
        continue

      products_buf = []
      buf_size = 250
      try:
        for prod in gs.list_products():
          bmc_product = None
          try:
            bmc_product = self.to_bmc_product(prod)
          except:
            pass

          if not bmc_product:
            continue

          products_buf.append(bmc_product)

          if len(products_buf) < buf_size:
            continue

          self.process_products(products_buf)
          products_buf = []

        if products_buf:
          self.process_products(products_buf)
          products_buf = []
      except Exception as e:
        logger.exception(e)

  def process_products(self, products):
    try:
      result = self.upload_to_bmc(products)
      logger.info(
        '[BmcUpload] ProductsCnt: %s, Succeed: %s, Failed: %s',
        len(products), result['succeed_cnt'], result['failed_cnt'])
    except Exception as e:
      logger.exception(e)

  def upload_to_bmc(self, bmc_products):
    return self.bs.upload_products(bmc_products)

  def to_bmc_product(self, product):
    if 'description' not in product or not product['description']:
      logger.info('[ProductNoDescription] %s', product)
      return

    if 'title' not in product or not product['title']:
      logger.info('[ProductNoTitle] %s', product)
      return

    if 'link' not in product or not product['link']:
      return

    prod = {
      'offerId': product['offerId'],
      'contentLanguage': product['contentLanguage'],
      'targetCountry': product['targetCountry'],
      'channel': 'online',
      'title': product['title'][:149],
      'description': product['description'][:9999],
      'availability': product['availability'],
      'price': {
        'value': product['price']['value'],
        'currency': product['price']['currency']
      },
      'expirationDate': self.expire_str,
      'link': product['link'],
      'condition': product.get('condition', 'new')
    }

    prod['imageLink'] = product['imageLink']
    if 'additionalImageLinks' in product and product['additionalImageLinks']:
      prod['additionalImageLinks'] = list(product['additionalImageLinks'])

    for attr in ['brand', 'mpn', 'gtin', 'color', 'ageGroup', 'gender', 'productTypes', 'customLabel0']:
      if attr in product and product[attr]:
        prod[attr] = product[attr]

    return prod
