import datetime

import dateutil
import dateutil.parser

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil


class SpapiUpdateItemOffersTask():
  def __init__(self, spapi, offer_service, marketplace, asins, condition='new', error_service=None):
    self.spapi = spapi
    self.error_service = error_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.condition = condition.lower()

  def run(self):
    offer_type = 'lowest_offer_listings'
    offers = self.spapi.get_item_offers_batch(self.marketplace, self.asins, self.condition)
    if not offers:
      if self.error_service:
        self.error_service.save_no_offer_asins(
          "offer", self.asins, self.marketplace
        )
      return

    self.offer_service.save_item_offers(
      offer_type, offers, self.marketplace, self.condition)
    logger.debug('[OfferSaved] %s', offers)
