# -*- coding: utf-8 -*-

import os
import datetime

from em_tasks import logger
from em_tasks.utils.product_service import ProductService


class AmzProductSeedsExporter():
  def __init__(self, marketplace, product_service, missing_info_output_path, missing_offer_output_path, days=None):
    self.product_service = product_service
    self.marketplace = marketplace.lower()
    self.condition = 'new'
    self.product_urls_indice_name = 'amz_products_list_{}'.format(self.marketplace)
    self.products_indice_name = 'amz_products_api_{}'.format(self.marketplace)
    self.offers_indice_name = 'lowest_offer_listings_{}_{}'.format(self.marketplace, self.condition)

    self.missing_info_output_path = os.path.abspath(os.path.expanduser(missing_info_output_path))
    missing_info_output_dir = os.path.dirname(self.missing_info_output_path)
    if not os.path.isdir(missing_info_output_dir):
      os.makedirs(missing_info_output_dir)
    self.missing_info_output_fh = open(self.missing_info_output_path, 'w+', encoding='utf-8', errors='ignore')

    self.missing_offer_output_path = os.path.abspath(os.path.expanduser(missing_offer_output_path))
    missing_offer_output_dir = os.path.dirname(self.missing_offer_output_path)
    if not os.path.isdir(missing_offer_output_dir):
      os.makedirs(missing_offer_output_dir)
    self.missing_offer_output_fh = open(self.missing_offer_output_path, 'w+', encoding='utf-8', errors='ignore')

    self.days = days

  def run(self):
    options = {'scroll_kwargs': {'filter_path': ['hits.hits._id', '_shards', '_scroll_id']}}
    if self.days:
      now = datetime.datetime.now()
      start_date_s = (now - datetime.timedelta(days=self.days)).date().strftime('%Y-%m-%d')
      options['query'] = {'query': {'bool': {'filter': [{'range': {'time': {'gte': start_date_s}}}]}}}

    try:
      prods_buf = []
      batch_size = 500
      for prod_id, _ in self.product_service.load_products(self.product_urls_indice_name, options):
        prods_buf.append(prod_id)
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.missing_info_output_fh.close()
      self.missing_offer_output_fh.close()

  def process_products(self, asins):
    asins_without_info = []
    asins_with_info = {}
    query = {"terms": {"_id": asins}}
    options = {'body': {'query': query}, '_source': False}
    prods_info = self.product_service.custom_search(self.products_indice_name, options)
    if prods_info:
      for item in prods_info['hits']['hits']:
        asins_with_info[item['_id']] = None
    for asin in asins:
      if asin in asins_with_info:
        continue

      asins_without_info.append(asin)

    if asins_without_info:
      self.missing_info_output_fh.write("\n".join(asins_without_info))
      self.missing_info_output_fh.write("\n")

    asins_without_offer = []
    offers = self.product_service.search_products(self.offers_indice_name, asins)
    if not offers:
      return

    for asin in asins:
      offer = offers.get(asin, None)
      if offer is None:
        asins_without_offer.append(asin)

    if asins_without_offer:
      self.missing_offer_output_fh.write("\n".join(asins_without_offer))
      self.missing_offer_output_fh.write("\n")
