# -*- coding: utf-8 -*-

import os
import datetime

from em_tasks import logger
from em_tasks.utils.product_service import ProductService


class EbayProductSeedsExporter():
  def __init__(self, marketplace, product_service, output_path, days=None):
    self.product_service = product_service
    self.marketplace = marketplace.lower()
    self.product_urls_indice_name = 'ebay_products_list_{}'.format(self.marketplace)
    self.products_indice_name = 'ebay_{}_products'.format(self.marketplace)
    self.days = days

    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

  def run(self):
    options = {'scroll_kwargs': {'filter_path': ['hits.hits._id', '_shards', '_scroll_id']}}
    if self.days:
      now = datetime.datetime.now()
      start_date_s = (now - datetime.timedelta(days=self.days)).date().strftime('%Y-%m-%d')
      options['query'] = {'query': {'bool': {'filter': [{'range': {'time': {'gte': start_date_s}}}]}}}

    try:
      prods_buf = []
      batch_size = 500
      for prod_id, _ in self.product_service.load_products(self.product_urls_indice_name, options):
        prods_buf.append(prod_id)
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.output_fh.close()

  def process_products(self, pids):
    pids_without_info = []
    pids_with_info = {}
    query = {"terms": {"_id": pids}}
    options = {'body': {'query': query}, '_source': False}
    prods_info = self.product_service.custom_search(self.products_indice_name, options)
    if prods_info:
      for item in prods_info['hits']['hits']:
        pids_with_info[item['_id']] = None
    for pid in pids:
      if pid in pids_with_info:
        continue

      pids_without_info.append(pid)

    if pids_without_info:
      self.output_fh.write("\n".join(pids_without_info))
      self.output_fh.write("\n")
