import datetime

from gspread.exceptions import APIError

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.store.models import *


class OrdersSheetBase():
  sheet_cols = {
    'AMZ': [
      'status', 'order-id', 'recipient-name', 'purchase-date', 'sku address', 'sku', 'price',
      'quantity-purchased', 'shipping-fee', 'ship-state', 'isbn-address', 'isbn', 'SELLER',
      'SELLER-ID', 'SELLER PRICE', 'URL', 'condition', 'character', 'Remark', 'reference',
      'CODE', '0', 'item-name', 'ship-address-1', 'ship-address-2', 'ship-city', 'ship-zip',
      'ship-phone-number', 'COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'ship-country',
      'SID', 'Sales Channel', 'Condition', 'Shipping Service', 'Expected Ship Date',
      'Estimated Delivery Date', 'Order Date', 'Supplier Estimated Delivery Date', 'Carrier',
      'Tracking Number', 'Note1', 'Note2', 'Note3'
    ],
    'Ebay': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date', 'sku',
      'item-name', 'options', 'sku address', 'isbn-address', 'price', 'quantity-purchased',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'Order Date', 'Remark', 'SELLER',
      'SELLER PRICE', 'Carrier', 'Tracking Number', 'Supplier Estimated Delivery Date', 'New link',
      'Ship to Customer', 'Gift', 'Operater'
    ],
    'Rakuten': [
      'status', 'order-id', 'recipient-name', 'purchase-date', 'sku address', 'sku', 'price',
      'quantity-purchased', 'shipping-fee', 'ship-state', 'isbn-address', 'isbn', 'SELLER',
      'SELLER-ID', 'SELLER PRICE', 'URL', 'condition', 'character', 'Remark', 'reference',
      'CODE', '0', 'item-name', 'ship-address-1', 'ship-address-2', 'ship-city', 'ship-zip',
      'ship-phone-number', 'COST', 'ORDER NUMBER', 'ACCOUNT', 'last code', 'ship-country',
      'SID', 'Sales Channel', 'Condition', 'Shipping Service', 'Expected Ship Date',
      'Estimated Delivery Date', 'Order Date', 'Supplier Estimated Delivery Date', 'Carrier',
      'Tracking Number', 'Note1', 'Note2', 'Note3'
    ],
    'Coupang': [
      'status', 'order-id', 'purchase-date', 'sku', 'sku address', 'price', 'quantity-purchased',
      'isbn-address', 'item-name', 'options', 'COST', 'Carrier', 'Tracking Number', 'Order Date',
      'ORDER NUMBER', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-country', 'ship-zip', 'ship-phone-number', 'Note1', 'Note2', 'Note3'
    ],
    'Naver': [
      'status', 'order-id', 'purchase-date', 'sku', 'sku address', 'price', 'quantity-purchased',
      'isbn-address', 'item-name', 'options', 'COST', 'Carrier', 'Tracking Number', 'Order Date',
      'ORDER NUMBER', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-country', 'ship-zip', 'ship-phone-number', 'Note1', 'Note2', 'Note3'
    ],
    '11ST': [
      'status', 'order-id', 'purchase-date', 'sku', 'sku address', 'price', 'quantity-purchased',
      'isbn-address', 'item-name', 'options', 'COST', 'Carrier', 'Tracking Number', 'Order Date',
      'ORDER NUMBER', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-country', 'ship-zip', 'ship-phone-number', 'Note1', 'Note2', 'Note3'
    ],
    'WEMAKEPRICE': [
      'status', 'order-id', 'purchase-date', 'sku', 'sku address', 'price', 'quantity-purchased',
      'isbn-address', 'item-name', 'options', 'COST', 'Carrier', 'Tracking Number', 'Order Date',
      'ORDER NUMBER', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-country', 'ship-zip', 'ship-phone-number', 'Note1', 'Note2', 'Note3'
    ],
    'EMP': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date', 'item-name',
      'sku address', 'sku', 'price', 'quantity-purchased', 'shipping-fee', 'isbn-address',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'Order Date', 'SELLER', 'SELLER PRICE', 'Carrier',
      'Tracking Number', 'Remark'
    ],
    'Dangdang': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date', 'item-name',
      'sku address', 'isbn-address', 'sku', 'price', 'quantity-purchased', 'shipping-fee',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'Order Date', 'SELLER', 'SELLER PRICE', 'Carrier',
      'Tracking Number', 'Remark'
    ],
    'Other': [
      'order-link', 'order-id', 'recipient-name', 'ship-address-1', 'ship-address-2', 'ship-city',
      'ship-state', 'ship-zip', 'ship-country', 'ship-phone-number', 'purchase-date',
      'sku address', 'sku', 'isbn-address', 'price', 'quantity-purchased', 'shipping-fee',
      'ORDER COST', 'ORDER NUMBER', 'ACCOUNT', 'Order Date', 'item-name', 'Carrier','Tracking Number', 'Remark'
    ]
  }

  spreadsheet_ids = {
    1: {
      'EMP': '1ZtOnb1fnmCT8pEg58WXmV0QlTb1VJYA8JVVzTJgyTFc',
      'Ebay': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'Ebay_US': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'AMZ_US': '1r_h4iU1iRovGG2XB9VIiQejtSY7O8nMdfro9jAvX1sU',
      'AMZ_UK': '1JyfmUPR4BIQXYwpPtNjFkAr9ac4kajak82CJHiq54dE',
      'AMZ_JP': '1TrUL9MUwjHaR-h4yIG6VCbIlaK87FZP08lijhxVELeM',
      'Coupang': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM',
      'Naver': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM',
      'Rakuten': '1sgwjRuw9ASeY6KDS94Xc4ke346mzN-welNHZS_au0bE',
      'Other': '12x27H7LpGkCtGtfI2UptjcFKkrbznrFMKYf2ZkANtO8',
      'Dangdang': '1yW9X4YtNNRcjd0aAQaoB7xcC1TlX3iLqpv8Ze_enFd4'
    },
    2: {
      'Coupang': '1DGM75rCW9-SFvEBOxCv9n-TnqHboFcRBH4UzWS9zUyk',
      'Naver': '1DGM75rCW9-SFvEBOxCv9n-TnqHboFcRBH4UzWS9zUyk',
      'AMZ_US': '1r_h4iU1iRovGG2XB9VIiQejtSY7O8nMdfro9jAvX1sU',
    },
    3: {
      'AMZ_US': '1zw7xzh1co7Gbgz1-eaoRCphp_bvZjb6PtqxaKSU96UI'
    },
    4: {
      'AMZ_JP': '1TrUL9MUwjHaR-h4yIG6VCbIlaK87FZP08lijhxVELeM',
      'Rakuten': '1TrUL9MUwjHaR-h4yIG6VCbIlaK87FZP08lijhxVELeM'
    },
    5: {
      'Coupang': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM',
      'Naver': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM',
      'WEMAKEPRICE': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM',
      '11ST': '1gIC4wB4U8Nwz945Oi6ed53zbtANH1hiIYzbXt-2U5NM'
    },
    6: {
      'Ebay': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw',
      'Ebay_US': '1-tSAKaS8KeIWf_TZcGEcc7T49JU9jWXm2HL2O3HTthw'
    },
    7: {
      'AMZ_JP': '1sgwjRuw9ASeY6KDS94Xc4ke346mzN-welNHZS_au0bE',
      'Rakuten': '1sgwjRuw9ASeY6KDS94Xc4ke346mzN-welNHZS_au0bE'
    },
    8: {
      'AMZ_UK': '1JyfmUPR4BIQXYwpPtNjFkAr9ac4kajak82CJHiq54dE'
    },
    9: {
      'AMZ_TR': '1WexuyXPkZo8t3j31W9-bow70UEc66RxnTpgtw6jluR4'
    },
    10: {
      'AMZ_UK': '1ilJhdGRS6mpG8bIRPGtU8C6mA8UpVQOIsWZ_dSwFY2Q',
      'AMZ_DE': '1xJ_sJ2SIzYh-dLHkIunp7b1tDOpStxrq1_6b39AKH0I'
    },
    11: {
      'Dangdang': '1yW9X4YtNNRcjd0aAQaoB7xcC1TlX3iLqpv8Ze_enFd4'
    },
    12: {
      'www_books_com_tw': '1yW9X4YtNNRcjd0aAQaoB7xcC1TlX3iLqpv8Ze_enFd4',
      'Www_books_com_tw': '1yW9X4YtNNRcjd0aAQaoB7xcC1TlX3iLqpv8Ze_enFd4'
    },
  }
