import json

from em_tasks.store.models import *
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.dangdang_product_formatter import DangdangProductFormatter

from em_tasks import logger


class ShopifyDangdangProductsExporter():
  def __init__(self, product_service, price_caculator, output_path):
    self.output_path = output_path
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')
    self.formater = DangdangProductFormatter(product_service, price_caculator)
    self.store_sellers = dict()
    self.batch_size = 1000

  def run(self):
    for store_seller in StoreSeller.select().execute():
      self.store_sellers[store_seller.id] = store_seller

    try:
      prods = dict()
      for prod in StoreUtil.get_dangdang_products():
        source_product_id = prod.get('source_product_id', None)
        if not source_product_id:
          continue

        prods[source_product_id] = prod
        if len(prods) < self.batch_size:
          continue

        self.process_products(prods)
        prods.clear()

      if prods:
        self.process_products(prods)
        prods.clear()
    finally:
      self.output_fh.close()

  def process_products(self, prods):
    source_product_ids = list(prods.keys())
    formatted_products = self.formater.format_products(source_product_ids)

    if not formatted_products:
      return

    for source_product_id, prod in prods.items():
      product = formatted_products.get(source_product_id, None)
      if not product:
        logger.info('[SourceProductNotFound] %s', source_product_id)
        continue

      if not product.get('existence', True):
        logger.info('[SourceProductNotExist] %s', source_product_id)
        continue

      store_seller = self.store_sellers.get(prod['store_seller_id'], None)
      if store_seller:
        product['seller'] = store_seller.code
      else:
        product['seller'] = 'em-cn'

      product['slug'] = prod['handle']

      self.output_fh.write(json.dumps(product, ensure_ascii=False))
      self.output_fh.write("\n")
