import json
import pdb

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class ShopifyLocationUpdater():
  def __init__(self, store_code, source, target_location_id):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    self.source = source.lower()
    self.target_location_id = target_location_id

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['name'], credential['token'], credential.get('release', None))

  def run(self):
    method = None
    if self.source == 'dangdang':
      method = 'get_dangdang_products'
    elif self.source == 'naver':
      method = 'get_naver_products'
    elif self.source == 'coupang':
      method = 'get_coupang_products'
    elif self.source == 'rakuten':
      method = 'get_rakuten_products'
    # elif self.source.find('ebay') != -1:
    #   method = 'get_ebay_products'
    # elif self.source.find('amz') != -1:
    #   method = 'get_amz_products'

    if not method:
      return

    batch_size = 250
    prod_sids = dict()
    for prod in getattr(StoreUtil, method)():
      prod_sids[prod['product_id']] = None
      if len(prod_sids) < batch_size:
        continue

      self.process_products(list(prod_sids.keys()))
      prod_sids.clear()

    if prod_sids:
      self.process_products(list(prod_sids.keys()))
      prod_sids.clear()

    self.pool.close()
    self.pool.join()

  def process_products(self, prod_sids):
    products = self.product_util.get_products(prod_sids)
    for pid, product in products.items():
      for variant in product.variants:
        self.product_util.relocate_inventory(self.target_location_id, variant.inventory_item_id, int(variant.inventory_quantity))

        logger.info('[InventoryRelocated] ProductID: %s, VariantID: %s', pid, variant.id)
