# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.store.store_util import StoreProduct


class ShopifyProductsIndexer():
  def __init__(self, product_service):
    self.product_service = product_service
    self.indice_name = 'shopify_products'

  def run(self):
    self.product_service.ensure_indice(self.indice_name)

    page = 1
    batch_size = 2500
    query = StoreProduct.select(StoreProduct.id).order_by(StoreProduct.id.desc())
    # query = StoreProduct.select(StoreProduct.id).order_by(StoreProduct.id.asc())
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    while page <= pages:
      product_ids = []
      for sp in query.paginate(page, batch_size).dicts():
        product_ids.append(sp['id'])

      products = []
      for sp in StoreProduct.select().where(StoreProduct.id.in_(product_ids)).dicts():
        products.append(sp)

      self.process_products(products)
      logger.debug('[ProductProcessed] Page: %s', page)

      page += 1

  def process_products(self, products):
    prod_buf = dict()
    pids = dict()
    for product in products:
      pid = '{}-{}'.format(product['source'], product['source_product_id'])
      pids[pid] = None

      product.pop('id', None)
      product.pop('store_seller_id', None)
      product['product_id'] = pid
      prod_buf[pid] = product

    prods = self.product_service.search_products(self.indice_name, list(pids.keys()))
    if prods:
      for pid in prods.keys():
        prod_buf.pop(pid, None)

    if prod_buf:
      prods = list(prod_buf.values())
      chunks = [prods[x:x + 500] for x in range(0, len(prods), 500)]
      for chunk in chunks:
        self.product_service.save_products(self.indice_name, chunk)
