# -*- coding: utf-8 -*-

import os
import json
import logging
import logging.handlers
import io
import pkg_resources
import datetime
import csv
import re
import ast

import click
from google.oauth2 import service_account
from google.cloud.translate_v2 import Client

from em_tasks import logger
from em_tasks.utils.product_service import ProductService


@click.command()
@click.option('-sa', '--service_account_path', type=str, help='Service account to authenticate google service.')
@click.argument('products_path')
def upload_products(service_account_path, products_path):
  if not products_path:
    logger.error('products_path is required!')
    return

  products_path = os.path.abspath(os.path.expanduser(products_path))
  files = []
  if os.path.isdir(products_path):
    for root, dirnames, filenames in os.walk(products_path):
      for filename in filenames:
        if filename.endswith(('.jsonl')):
          files.append(os.path.join(root, filename))
  else:
    if not os.path.isfile(products_path):
      logger.error('Could not find products file - %s', products_path)
      return

    files.append(products_path)

  if not service_account:
    logger.error("Service account is required to authenticate google service")
    return

  service_account_path = os.path.abspath(os.path.expanduser(service_account_path))
  if not os.path.isfile(service_account_path):
    logger.error("Could not find service account file - %s", service_account_path)
    return

  target_language = 'en'
  translation_service = Client(
    target_language=target_language,
    credentials=service_account.Credentials.from_service_account_file(service_account_path))

  indice_name = 'naver_products'

  ps = ProductService(
    '35.192.187.156', port=80, user='elasticuser', password='KbersRiseUp153')
  ps.ensure_indice(indice_name)

  products_buf = []
  batch_size = 50
  for prods_path in files:
    logger.info('[ProcessingFile] %s', prods_path)
    with open(prods_path, encoding='utf-8', errors='ignore', newline='') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        record = ast.literal_eval(s)
        if record['variants']:
          v_titles = [v['title'] for v in record['variants']]
          if len(v_titles) != len(set(v_titles)):
            logger.info("[MultipleOptionsProduct] %s", record['url'])
            continue

        prod = dict(record)
        prod['sku'] = 'X10-{}'.format(prod['product_id'])
        prod['shipping_days_min'] = None
        prod['shipping_days_max'] = None
        if not prod['description'] and prod.get('specifications', []):
          specs = []
          for spec in prod.get('specifications', []):
            specs.append('<li>{}: {}</li>'.format(spec['name'], spec['value']))
          prod['description'] = '<ul>{}</ul>'.format(''.join(specs))

        variants = prod.pop('variants')
        if variants:
          prod['variants'] = []
          option_name = 'Option'
          if not prod['options']:
            prod['options'] = [{'name': option_name}]

          for variant in variants:
            prod['variants'].append({
              'sku': 'X10-{}-{}'.format(prod['product_id'], variant['id']),
              'price': variant['price'],
              'available_qty': variant['available_qty'],
              'option_values': [{'option_name': option_name, 'option_value': variant['title']}],
              'barcode': None
            })

        products_buf.append(prod)
        if len(products_buf) < batch_size:
          continue

        texts = []
        for prod in products_buf:
          texts.append(prod['title'])
          texts.append(prod['description'] or '')

        result = None
        try:
          result = translation_service.translate(texts)
        except ValueError:
          pass
        except Exception as e:
          logger.exception(e)
        for idx, product in enumerate(products_buf):
          if result:
            try:
              product['title_en'] = result[2 * idx]['translatedText']
              product['description'] = result[2 * idx + 1]['translatedText']
            except Exception as e:
              logger.exception(e)

        try:
          ps.save_products(indice_name, products_buf)
        except Exception as e:
          logger.exception(e)

        products_buf = []

      if products_buf:
        texts = []
        for prod in products_buf:
          texts.append(prod['title'])
          texts.append(prod['description'] or '')

        result = None
        try:
          result = translation_service.translate(texts)
        except ValueError:
          pass
        except Exception as e:
          logger.exception(e)
        for idx, product in enumerate(products_buf):
          if result:
            try:
              product['title_en'] = result[2 * idx]['translatedText']
              product['description'] = result[2 * idx + 1]['translatedText']
            except Exception as e:
              logger.exception(e)

        try:
          ps.save_products(indice_name, products_buf)
        except Exception as e:
          logger.exception(e)

        products_buf = []


if __name__ == '__main__':
  upload_products()
