# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class NaverUploadableProductsFilter():
  def __init__(self, product_service):
    self.product_service = product_service
    self.indice_name = 'naver_products'

  def filter(self):
    blacklisted_path = './naver_blacklisted.txt'
    records = dict()
    stats = dict()
    for keyword in BlacklistKeyword.select().order_by(BlacklistKeyword.id.desc()).dicts():
      k = keyword['keywords']
      stats[k]= 0

      query = {
        'query': {
          'query': {
            'multi_match': {
              'query': k,
              'type': 'phrase',
              'fields': ['title_en^10', 'description']
            }
          }
        }
      }
      for prod_id, _ in self.product_service.load_products(self.indice_name, query):
        records[prod_id] = None
        stats[k] += 1

    total_products_cnt = 0
    with open('./naver_blacklisted.txt', 'w+', encoding='utf-8', errors='ignore') as fh:
      fh.write("\n".join(records.keys()))
    for k, products_cnt in stats.items():
      if not products_cnt:
        continue

      total_products_cnt += products_cnt

      logger.debug('[BlacklistedProducts] Keyword: %s, ProductCnt: %s', k, products_cnt)

    logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', total_products_cnt)

    uploaded_prods = dict()
    for prod in StoreUtil.get_naver_products():
      uploaded_prods[prod['source_product_id']] = None

    logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(uploaded_prods))

    for prod_id, prod in self.product_service.load_products(self.indice_name):
      exist = prod.get('existence', True)
      if not exist:
        continue

      if prod_id in records or prod_id in uploaded_prods:
        continue

      yield (prod_id, prod)
