from em_tasks.utils.pipelines.base import BasePipeline


class PriceCalculatorPipeline(BasePipeline):
  def __init__(self, price_calc):
    self.price_calc = price_calc

  def execute(self, product):
    price = product.get('price', 0) or 0
    offer = self.calc_offer(product)
    if offer:
      product['cost_price'] = self.price_calc.calc_cost(product)
      product['price'] = offer.get('price', 0)
      product['quantity'] = offer.get('quantity', 0)
      product['currency'] = 'USD'
    else:
      product['cost_price'] = self.price_calc.calc_cost(product)
      product['price'] = 0
      product['quantity'] = 0

    shipping_fee = product.get('shipping_fee', 0) or 0
    variants = product.get('variants', None)
    if not variants:
      return product

    for variant in product['variants']:
      price = variant.get('price', 0) or 0
      variant['shipping_fee'] = shipping_fee
      offer = self.calc_offer(variant)
      if offer:
        variant['cost_price'] = self.price_calc.calc_cost(variant)
        variant['price'] = offer.get('price', 0)
        variant['quantity'] = offer.get('quantity', 0)
        variant['currency'] = 'USD'
      else:
        variant['cost_price'] = self.price_calc.calc_cost(variant)
        variant['price'] = 0
        variant['quantity'] = 0

    return product

  def calc_offer(self, item):
    product_price = item.get('price', 0) or 0
    shipping_fee = item.get('shipping_fee', 0) or 0
    price = product_price + shipping_fee
    currency = item.get('currency', 'USD')
    src_offer = {'price': price, 'currency': currency}
    return self.price_calc.calc_offer(src_offer)
