from em_tasks.utils.pipelines.base import BasePipeline

from em_tasks import logger


class PriceFilterPipeline(BasePipeline):
  def __init__(self, price_max=300, price_min=1):
    self.price_max = price_max
    self.price_min = price_min

  def execute(self, product):
    result = self.check_product(product)
    passed = result.get('passed', True)
    if passed:
      return product

  def check_product(self, product):
    price = product.get('price', None)
    if not price:
      return {'passed': False, 'reason': '[NoPrice]'}

    if price > self.price_max:
      return {'passed': False, 'reason': '[PriceTooHigh] Price: {}, MaxPrice: {}'.format(price, self.price_max)}

    if price < self.price_min:
      return {'passed': False, 'reason': '[PriceTooLow] Price: {}, MinPrice: {}'.format(price, self.price_min)}

    return {'passed': True, 'reason': ''}
