import os
import time

from em_tasks.utils.product_loaders.base import BaseProductLoader


class EsProductLoader(BaseProductLoader):
  def __init__(self, product_service, indice_name, product_ids_path = None):
    self.product_service = product_service
    self.indice_name = indice_name
    if product_ids_path:
      self.product_ids_path = os.path.abspath(os.path.expanduser(product_ids_path))
    else:
      self.product_ids_path = None

  def load(self):
    if self.product_ids_path is None:
      for product_id, product in self.product_service.load_products(self.indice_name):
        yield product
    else:
      batch_size = 500
      product_ids_buf = []
      with open(self.product_ids_path, encoding='utf-8', errors='ignore') as fh:
        for line in fh:
          s = line.strip()
          if not s:
            continue

          product_id = s
          product_ids_buf.append(product_id)
          if len(product_ids_buf) < batch_size:
            continue

          products = self.get_products(product_ids_buf)
          if not products:
            continue

          for _, product in products.items():
            yield product

          product_ids_buf = []

      if product_ids_buf:
        products = self.get_products(product_ids_buf)
        if not products:
          return

        for _, product in products.items():
          yield product

        product_ids_buf = []

  def get_products(self, product_ids):
    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    return products
