import json
import pdb

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class ShopifyAmzRepricer():
  def __init__(self, store_code, marketplace, offer_service, price_caculator):
    self.condition = 'new'

    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['name'], credential['token'], credential.get('release', None))

    self.marketplace = marketplace
    self.offer_service = offer_service
    self.price_calc = price_caculator

    self.gss = dict()
    gmc_stores = GmcStore.select().execute()
    for gmc_store in gmc_stores:
      self.gss[gmc_store.id] = GoogleShopping(
        gmc_store.merchant_id, json.loads(gmc_store.service_account_data))

  def run(self):
    batch_size = 250
    prods = dict()
    for amz_prod in StoreUtil.get_amz_products(self.marketplace):
      prods[amz_prod['product_id']] = amz_prod
      if len(prods) < batch_size:
        continue

      self.process_products(prods)
      prods.clear()

    if prods:
      self.process_products(prods)
      prods.clear()

  def process_products(self, prods):
    offers = dict()
    asins = [prod['asin'] for prod in prods.values()]
    src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
    if not src_offers:
      return

    for asin, src_offer in src_offers.items():
      offers[asin] = {'asin': asin, 'src_offer': src_offer}
      offer = self.price_calc.calc_offer(src_offer)
      if offer:
        offers[asin].update(offer)
      else:
        offers[asin] = offer

    for _, prod in prods.items():
      asin = prod['asin']
      if asin in offers:
        offer = offers[asin]
        prod['offer'] = offer

    store_offers = self.product_util.set_products_offer(prods)
    try:
      self.update_gmc_offers(store_offers)
    except Exception as e:
      logger.exception(e)

  def update_gmc_offers(self, store_offers):
    items = dict()
    for _, store_offer in store_offers.items():
      if not store_offer['handle']:
        continue

      items[store_offer['handle']] = store_offer

    handles = list(items.keys())
    gmc_offers = dict()
    for gmc_product in GmcProduct.select().where(GmcProduct.handle.in_(handles)).execute():
      if gmc_product.handle not in items:
        continue

      store_offer = None
      store_price = 0
      store_availability = 'out of stock'
      v_offers = list(items[gmc_product.handle]['offers'].values())
      if v_offers:
        store_offer = v_offers[0]
        store_price = round(store_offer['price'], 2)
        store_availability = 'in stock' if store_offer['quantity'] > 0 else 'out of stock'

      gmc_price = round(gmc_product.price, 2)
      gmc_availability = gmc_product.availability
      if gmc_availability != store_availability or gmc_price != store_price:
        if gmc_product.gmc_store_id not in gmc_offers:
          gmc_offers[gmc_product.gmc_store_id] = []

        gmc_offers[gmc_product.gmc_store_id].append({
          'productId': gmc_product.gmc_id,
          'price': {'currency': 'USD', 'value': store_price},
          'availability': store_availability
        })

    for gmc_store_id, gmc_store_offers in gmc_offers.items():
      if gmc_store_id not in self.gss:
        continue

      if not gmc_store_offers:
        continue

      gs = self.gss[gmc_store_id]
      result = gs.update_inventory(gmc_store_offers)
      logger.info(
          '[GoogleMerchantInventoryUpdate] Succeed: %s, Failed: %s',
          result['succeed_cnt'], result['failed_cnt'])
