import json
import math
import pdb

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *


class ShopifyCustomLabelUpdater():
  def __init__(self):
    self.gss = dict()
    self.merchant_ids = dict()
    gmc_stores = GmcStore.select().execute()
    for gmc_store in gmc_stores:
      if gmc_store.merchant_id == '654568556':
        continue

      self.merchant_ids[gmc_store.merchant_id] = gmc_store.id
      self.gss[gmc_store.id] = GoogleShopping(
        gmc_store.merchant_id, json.loads(gmc_store.service_account_data))

  def run(self):
    batch_size = 5000
    gps = dict()
    for merchant_id, gmc_store_id in self.merchant_ids.items():
      if merchant_id == '654568556':
        continue

      page = 1
      query = GmcProduct.select().where(GmcProduct.gmc_store_id == gmc_store_id).order_by(GmcProduct.id.asc())
      total_cnt = query.count()
      pages = int((total_cnt + batch_size) / batch_size)
      while page <= pages:
        for gp in query.paginate(page, batch_size).execute():
          gps[gp.offer_id] = gp

        self.process_products(merchant_id, gps)
        gps.clear()

        page += 1

      if gps:
        self.process_products(merchant_id, gps)
        gps.clear()

  def process_products(self, merchant_id, gps):
    custom_labels = dict()

    offer_ids = list(gps.keys())
    gaps = dict()
    for gap in GoogleAdsPerformance.select().where((GoogleAdsPerformance.merchant_id == merchant_id) & (GoogleAdsPerformance.item_id.in_(offer_ids))).dicts():
      gaps[gap['item_id']] = gap

    for item_id, gp in gps.items():
      prod_id = gp.gmc_id
      if gp.price < 10:
        custom_label0 = 'Price_<10'
      elif gp.price < 100:
        start = math.floor(gp.price / 10) * 10
        end = math.ceil(gp.price / 10) * 10
        if start == end:
          end = start + 10
        custom_label0 = 'Price_{}-{}'.format(start, end)
      elif gp.price < 300:
        base = gp.price - 100
        start = math.floor(base / 30) * 30 + 100
        end = math.ceil(base / 30) * 30 + 100
        if start == end:
          end = start + 30
        custom_label0 = 'Price_{}-{}'.format(start, end)
      else:
        custom_label0 = 'Price_>300'

      performance = gaps.get(item_id, None)
      if performance:
        custom_label1 = '0'
        if performance['impressions'] <= 0 and performance['clicks'] <= 0:
          custom_label1 = 'IMP0_CLK0'
        elif performance['impressions'] > 0 and performance['clicks'] <= 0:
          custom_label1 = 'IMP_CLK0'
        elif performance['impressions'] >= 1000 and performance['clicks'] <= 0:
          custom_label1 = 'IMP>1000_CLK0'
        elif performance['impressions'] >= 1000 and performance['clicks'] <= 10 and performance['conversions'] <= 0:
          custom_label1 = 'IMP>1000_CLK<10_CVN0'
        elif performance['impressions'] >= 1000 and performance['clicks'] > 10 and performance['conversions'] <= 0:
          custom_label1 = 'IMP>1000_CLK>10_CVN0'
        else:
          custom_label1 = 'IMP_CLK'

        if performance['cost'] <= 0:
          custom_label2 = 'Cost_0'
        elif performance['cost'] <= 10:
          custom_label2 = 'Cost_0-10'
        elif performance['cost'] <= 20:
          custom_label2 = 'Cost_10-20'
        elif performance['cost'] <= 30:
          custom_label2 = 'Cost_20-30'
        elif performance['cost'] <= 40:
          custom_label2 = 'Cost_30-40'
        elif performance['cost'] <= 50:
          custom_label2 = 'Cost_40-50'
        else:
          custom_label2 = 'Cost_>50'

        if performance['conversions'] <= 0:
          custom_label3 = 'Conversion_0'
        else:
          custom_label3 = 'Conversion_{}'.format(performance['conversions'])

        if performance['roas'] <= 0:
          custom_label4 = 'ROAS_0'
        elif performance['roas'] >= 7:
          custom_label4 = 'ROAS_>700'
        else:
          start = math.floor(performance['roas']) * 100
          end = math.ceil(performance['roas']) * 100
          if start == end:
            end = start + 100
          custom_label4 = 'ROAS_{}-{}'.format(start, end)

        custom_labels[item_id] = {
          'productId': prod_id,
          'customLabel0': custom_label0,
          'customLabel1': custom_label1,
          'customLabel2': custom_label2,
          'customLabel3': custom_label3,
          'customLabel4': custom_label4,
        }
      else:
        custom_labels[item_id] = {
          'productId': prod_id,
          'customLabel0': custom_label0,
          'customLabel1': 'IMP0_CLK0',
          'customLabel2': 'Cost_0',
          'customLabel3': 'Conversion_0',
          'customLabel4': 'ROAS_0',
        }

    self.update_gmc_custom_label(merchant_id, custom_labels)

  def update_gmc_custom_label(self, merchant_id, custom_labels):
    if merchant_id not in self.merchant_ids:
      return

    gmc_store_id = self.merchant_ids[merchant_id]
    items = list(custom_labels.values())
    n = 250
    chunks = [items[x:x + n] for x in range(0, len(items), n)]
    for chunk in chunks:
      gs = self.gss[gmc_store_id]
      result = gs.update_products(chunk)

      logger.info(
        '[GoogleMerchantCustomLabelUpdate] Succeed: %s, Failed: %s',
        result['succeed_cnt'], result['failed_cnt'])
