import json
import pdb
import multiprocessing
from multiprocessing.pool import ThreadPool

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.shopify_base_repricer import ShopifyBaseRepricer
from em_tasks.store.models import *


class ShopifyDangdangRepricer(ShopifyBaseRepricer):
  def __init__(self, store_code, product_service, price_calculator):
    super(ShopifyDangdangRepricer, self).__init__()

    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['name'], credential['token'], credential.get('release', None))

    self.product_service = product_service
    self.price_calc = price_calculator
    self.indice_name = 'dangdang_products'
    self.pool = ThreadPool(3)

  def run(self):
    batch_size = 250
    prods = dict()
    for dangdang_prod in StoreUtil.get_dangdang_products():
      prods[dangdang_prod['source_product_id']] = dangdang_prod
      if len(prods) < batch_size:
        continue

      self.process_products(prods)
      prods.clear()

    if prods:
      self.process_products(prods)
      prods.clear()

    self.pool.close()
    self.pool.join()

  def process_products(self, prods):
    product_ids = list(prods.keys())
    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      if pid not in prods:
        continue

      prod = prods[pid]

      src_offer = None

      existence = product.get('existence', True)
      has_only_default_variant = product.get('has_only_default_variant', True)
      src_product_price = round(product.get('price', 0) / 2.5, 2)
      src_shipping_fee = product.get('shipping_fee', 0) or 0
      currency = product.get('currency', None) or 'USD'
      if not existence or not has_only_default_variant or not src_product_price:
        prod['offer'] = {'price': 0, 'quantity': 0}
      else:
        src_offer = {'price': src_product_price + src_shipping_fee, 'currency': currency}

        offer = self.price_calc.calc_offer(src_offer)
        prod['offer']= offer

    offers = dict()
    for _, prod in prods.items():
      offers[prod['product_id']] = prod

    if offers:
      store_offers = self.product_util.set_products_offer(offers, self.pool)
      try:
        self.update_gmc_offers(store_offers)
      except Exception as e:
        logger.exception(e)
        handles = [store_offer['handle'] for store_offer in store_offers.values() if store_offer['handle']]
        logger.warning('[Handles] %s', handles)
