# -*- coding: utf-8 -*-

import datetime
import json
import time
import re

import click
import dateparser
import shopify
import shopify_python_api_patcher
from shopify_python_api_patcher.utils import get_current_release

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.shopify_base_repricer import ShopifyBaseRepricer


class ShopifyRakutenRepricer(ShopifyBaseRepricer):
  def __init__(self, store_code, product_service, price_calculator):
    super(ShopifyRakutenRepricer, self).__init__()
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    store_name = credential['name']
    app_token = credential['token']
    release = get_current_release()
    shop_url = "{}.myshopify.com".format(store_name)

    session = shopify.Session(shop_url, release, app_token)
    shopify.ShopifyResource.activate_session(session)
    self.shop = shopify.Shop.current()

    self.product_service = product_service
    self.price_calc = price_calculator
    self.indice_name = 'rakuten_products'

  def run(self):
    batch_size = 250
    products = dict()
    for rakuten_prod in StoreUtil.get_rakuten_products():
      products[rakuten_prod['product_id']] = rakuten_prod

      if len(products) < batch_size:
        continue

      self.process_products(products)
      products.clear()

    if products:
      self.process_products(products)
      products.clear()

  def process_products(self, prods):
    store_offers = dict()
    product_ids = ','.join([str(pid) for pid in list(prods.keys())])
    sps = shopify.Product.find(ids=product_ids, limit=len(prods))
    for sp in sps:
      store_offer = {'handle': sp.handle, 'product_id': sp.id, 'offers': {}}
      variant = sp.variants[0]
      for variant in sp.variants:
        store_offer['offers'][variant.id] = {
          'price': round(float(variant.price), 2),
          'quantity': int(variant.inventory_quantity)
        }
      store_offers[sp.id] = store_offer

    if store_offers:
      self.update_gmc_offers(store_offers)
