import json
import pdb

from em_tasks import logger
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class SpreeAmzFilter():
  def __init__(self, store_code, marketplace, product_service, output_path):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.marketplace = marketplace.lower()
    self.product_service = product_service
    self.indice_name = 'amz_products_api_{}'.format(self.marketplace)
    if self.marketplace == 'uk':
      self.filter = AmzProductsFilterPipeline(self.marketplace, 2.2, 11.8)
    else:
      self.filter = AmzProductsFilterPipeline(self.marketplace, 2.5, 30)

    self.output_fh = open(output_path, 'w+', encoding='utf-8', errors='ignore')

  def run(self):
    batch_size = 250
    prods = dict()
    try:
      for amz_prod in self.product_util.get_amz_products(self.marketplace):
        prods[amz_prod['product_id']] = amz_prod
        if len(prods) < batch_size:
          continue

        self.process_products(prods)
        prods.clear()

      if prods:
        self.process_products(prods)
        prods.clear()
    finally:
      self.output_fh.close()

  def process_products(self, prods):
    offers = dict()
    asins = [prod['source_product_id'] for prod in prods.values()]
    products = self.product_service.search_products(self.indice_name, asins)
    if not products:
      return

    not_passed_asins = dict()
    for asin, product in products.items():
      passed = self.filter.execute(product)
      if not passed:
        not_passed_asins[asin] = None

    not_passed_prods = []
    for _, prod in prods.items():
      asin = prod['source_product_id']
      if asin in not_passed_asins:
        not_passed_prods.append(prod['product_id'])

    if not_passed_prods:
      self.output_fh.write("\n".join(not_passed_prods))
      self.output_fh.write("\n")
