import json
import multiprocessing
from multiprocessing.pool import ThreadPool
import pdb

import sentry_sdk

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.utils.spree.base_repricer import SpreeBaseRepricer
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class SpreeAmzRepricer(SpreeBaseRepricer):
  def __init__(self, store_code, src_marketplace, offer_service, price_caculator, expired_callbacks):
    super(SpreeAmzRepricer, self).__init__()

    self.condition = 'new'

    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
    # self.product_util = ProductUtil('http://localhost:3000', credential['api_key'], credential.get('api_version', 'v1'))

    self.src_marketplace = src_marketplace.lower()
    self.offer_service = offer_service
    self.price_calc = price_caculator
    self.expired_callbacks = expired_callbacks
    self.stats = {'products_cnt': 0, 'expired_cnt': 0, 'failed_cnt': 0, 'in_stock': 0, 'out_of_stock': 0}

  def run(self):
    batch_size = 250
    prods = dict()
    for amz_prod in self.product_util.get_amz_products(self.src_marketplace):
      self.stats['products_cnt'] += 1

      prods[amz_prod['product_id']] = amz_prod
      if len(prods) < batch_size:
        continue

      self.process_products(prods)
      prods.clear()

    if prods:
      self.process_products(prods)
      prods.clear()

  def get_stats(self):
    return self.stats

  def process_products(self, prods):
    offers = dict()
    expired_asins = []

    asins = [prod['source_product_id'] for prod in prods.values()]
    src_offers = self.offer_service.get_offers(self.src_marketplace, asins, self.condition)
    if not src_offers:
      payload = {'marketplace': self.src_marketplace, 'asins': asins, 'condition': self.condition}
      try:
        sentry_sdk.capture_message('[NoOffers] {}'.format(json.dumps(payload)))
      except:
        pass

      self.stats['failed_cnt'] += 1

      return

    for asin, src_offer in src_offers.items():
      if src_offer:
        expired = src_offer.get('expired', False)
        if expired:
          expired_asins.append(asin)
          self.stats['expired_cnt'] += 1
      else:
        logger.debug('[NoSrcOffer] Marketplace: %s, ASIN: %s', self.src_marketplace, asin)

      offers[asin] = {'asin': asin, 'src_offer': src_offer}
      offer = self.price_calc.calc_offer(src_offer)
      if offer:
        offers[asin].update(offer)
        if offer.get('quantity', 0) > 0 and offer.get('price', 0) > 0:
          self.stats['in_stock'] += 1
        else:
          self.stats['out_of_stock'] += 1
      else:
        offers[asin] = offer
        self.stats['failed_cnt'] += 1

    for _, prod in prods.items():
      asin = prod['source_product_id']
      if asin in offers:
        offer = offers[asin]
        prod['offer'] = offer

    try:
      store_offers = self.product_util.set_products_offer(prods, None)
    except Exception as e:
      try:
        sentry_sdk.capture_exception(e)
      except:
        pass

    if expired_asins and self.expired_callbacks:
      for callback in self.expired_callbacks:
        try:
          callback(expired_asins)
        except:
          pass
