# -*- coding: utf-8 -*-

import os
import json
import logging
import logging.handlers
import io
import pkg_resources
import datetime
import csv
import re
import ast

import click

from em_tasks import logger
from em_tasks.utils.product_service import ProductService


@click.command()
@click.argument('products_path')
def upload_products(products_path):
  if not products_path:
    logger.error('products_path is required!')
    return

  products_path = os.path.abspath(os.path.expanduser(products_path))
  files = []
  if os.path.isdir(products_path):
    for root, dirnames, filenames in os.walk(products_path):
      for filename in filenames:
        if filename.endswith(('.jsonl')):
          files.append(os.path.join(root, filename))
  else:
    if not os.path.isfile(products_path):
      logger.error('Could not find products file - %s', products_path)
      return

    files.append(products_path)

  indice_name = 'www_books_com_tw_products'

  ps = ProductService(
    '35.192.187.156', port=80, user='elasticuser', password='KbersRiseUp153')
  ps.ensure_indice(indice_name)

  products_buf = []
  batch_size = 50
  for prods_path in files:
    logger.info('[ProcessingFile] %s', prods_path)
    with open(prods_path, encoding='utf-8', errors='ignore', newline='') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        record = json.loads(s)
        prod = dict(record)
        prod['sku'] = 'X11-{}'.format(prod['product_id'])
        img_urls = []
        for img in prod['images'].split(';'):
          img_urls.append(img.split('&').pop(0))
        prod['images'] = ';'.join(img_urls)

        specifications = prod.get('specifications', [])
        upc = prod['upc']
        if not upc:
          for spec in specifications:
            if spec['name'].lower().find('isbn') != -1:
              upc = spec['value']
              break

        brand = prod.get('brand', None)
        if not brand:
          for spec in specifications:
            if spec['name'] == '出版社':
              prod['brand'] = spec['value']
              break

        products_buf.append(prod)
        if len(products_buf) < batch_size:
          continue

        try:
          ps.save_products(indice_name, products_buf)
        except Exception as e:
          logger.exception(e)

        products_buf = []

      if products_buf:
        try:
          ps.save_products(indice_name, products_buf)
        except Exception as e:
          logger.exception(e)

        products_buf = []


if __name__ == '__main__':
  upload_products()
